@@ -91,6 +91,7 @@ import re
 import select
 import signal
 from time import sleep
+from itertools import chain
 
 from mininet.cli import CLI
 from mininet.log import info, error, debug, output
@@ -101,7 +102,7 @@ from mininet.util import macColonHex, ipStr, ipParse, netParse, ipAdd
 from mininet.term import cleanUpScreens, makeTerms
 
 # Mininet version: should be consistent with README and LICENSE
-VERSION = "2.0.0"
+VERSION = "2.1.0"
 
 class Mininet( object ):
     "Network emulation with hosts spawned in network namespaces."
@@ -208,16 +209,26 @@ class Mininet( object ):
     def addController( self, name='c0', controller=None, **params ):
         """Add controller.
            controller: Controller class"""
+        # Get controller class
         if not controller:
             controller = self.controller
-        controller_new = controller( name, **params )
+        # Construct new controller if one is not given
+        if isinstance(name, Controller):
+            controller_new = name
+            # Pylint thinks controller is a str()
+            # pylint: disable=E1103
+            name = controller_new.name
+            # pylint: enable=E1103
+        else:
+            controller_new = controller( name, **params )
+        # Add new controller to net
         if controller_new:  # allow controller-less setups
             self.controllers.append( controller_new )
             self.nameToNode[ name ] = controller_new
         return controller_new
 
-    # BL: is this better than just using nameToNode[] ?
-    # Should it have a better name?
+    # BL: We now have four ways to look up nodes
+    # This may (should?) be cleaned up in the future.
     def getNodeByName( self, *args ):
         "Return node(s) with given name(s)"
         if len( args ) == 1:
@@ -228,6 +239,37 @@ class Mininet( object ):
         "Convenience alias for getNodeByName"
         return self.getNodeByName( *args )
 
+    # Even more convenient syntax for node lookup and iteration
+    def __getitem__( self, key ):
+        """net [ name ] operator: Return node(s) with given name(s)"""
+        return self.nameToNode[ key ]
+
+    def __iter__( self ):
+        "return iterator over node names"
+        for node in chain( self.hosts, self.switches, self.controllers ):
+            yield node.name
+
+    def __len__( self ):
+        "returns number of nodes in net"
+        return ( len( self.hosts ) + len( self.switches ) +
+                 len( self.controllers ) )
+
+    def __contains__( self, item ):
+        "returns True if net contains named node"
+        return item in self.nameToNode
+
+    def keys( self ):
+        "return a list of all node names or net's keys"
+        return list( self )
+
+    def values( self ):
+        "return a list of all nodes or net's values"
+        return [ self[name] for name in self ]
+
+    def items( self ):
+        "return (key,value) tuple list for every node in net"
+        return zip( self.keys(), self.values() )
+
     def addLink( self, node1, node2, port1=None, port2=None,
                  cls=None, **params ):
         """"Add a link from node1 to node2
@@ -250,7 +292,7 @@ class Mininet( object ):
             info( host.name + ' ' )
             intf = host.defaultIntf()
             if intf:
-                host.configDefault( defaultRoute=intf )
+                host.configDefault()
             else:
                 # Don't configure nonexistent intf
                 host.configDefault( ip=None, mac=None )
@@ -275,7 +317,7 @@ class Mininet( object ):
 
         info( '*** Creating network\n' )
 
-        if not self.controllers:
+        if not self.controllers and self.controller:
             # Add a default controller
             info( '*** Adding controller\n' )
             classes = self.controller
@@ -325,6 +367,9 @@ class Mininet( object ):
 
     def startTerms( self ):
         "Start a terminal for each node."
+        if 'DISPLAY' not in os.environ:
+            error( "Error starting terms: Cannot connect to display\n" )
+            return
         info( "*** Running terms on %s\n" % os.environ[ 'DISPLAY' ] )
         cleanUpScreens()
         self.terms += makeTerms( self.controllers, 'controller' )
@@ -362,16 +407,16 @@ class Mininet( object ):
         if self.terms:
             info( '*** Stopping %i terms\n' % len( self.terms ) )
             self.stopXterms()
-        info( '*** Stopping %i hosts\n' % len( self.hosts ) )
-        for host in self.hosts:
-            info( host.name + ' ' )
-            host.terminate()
-        info( '\n' )
         info( '*** Stopping %i switches\n' % len( self.switches ) )
         for switch in self.switches:
             info( switch.name + ' ' )
             switch.stop()
         info( '\n' )
+        info( '*** Stopping %i hosts\n' % len( self.hosts ) )
+        for host in self.hosts:
+            info( host.name + ' ' )
+            host.terminate()
+        info( '\n' )
         info( '*** Stopping %i controllers\n' % len( self.controllers ) )
         for controller in self.controllers:
             info( controller.name + ' ' )
@@ -458,27 +503,39 @@ class Mininet( object ):
                     lost += sent - received
                     output( ( '%s ' % dest.name ) if received else 'X ' )
             output( '\n' )
+        if packets > 0:
             ploss = 100 * lost / packets
-        output( "*** Results: %i%% dropped (%d/%d lost)\n" %
-                ( ploss, lost, packets ) )
+            received = packets - lost
+            output( "*** Results: %i%% dropped (%d/%d received)\n" %
+                    ( ploss, received, packets ) )
+        else:
+            ploss = 0
+            output( "*** Warning: No packets sent\n" )
         return ploss
 
     @staticmethod
     def _parsePingFull( pingOutput ):
         "Parse ping output and return all data."
+        errorTuple = (1, 0, 0, 0, 0, 0)
         # Check for downed link
-        if 'connect: Network is unreachable' in pingOutput:
-            return (1, 0)
+        r = r'[uU]nreachable'
+        m = re.search( r, pingOutput )
+        if m is not None:
+            return errorTuple
         r = r'(\d+) packets transmitted, (\d+) received'
         m = re.search( r, pingOutput )
         if m is None:
             error( '*** Error: could not parse ping output: %s\n' %
                    pingOutput )
-            return (1, 0, 0, 0, 0, 0)
+            return errorTuple
         sent, received = int( m.group( 1 ) ), int( m.group( 2 ) )
         r = r'rtt min/avg/max/mdev = '
         r += r'(\d+\.\d+)/(\d+\.\d+)/(\d+\.\d+)/(\d+\.\d+) ms'
         m = re.search( r, pingOutput )
+        if m is None:
+            error( '*** Error: could not parse ping output: %s\n' %
+                   pingOutput )
+            return errorTuple
         rttmin = float( m.group( 1 ) )
         rttavg = float( m.group( 2 ) )
         rttmax = float( m.group( 3 ) )
